"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _utils = require("./utils");

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Configuration handler to manipulate build configurations for project and targets
 */
var Configuration = function () {
    function Configuration(project, xcode) {
        _classCallCheck(this, Configuration);

        this.project = project;
        this.xcode = xcode;
    }

    /**
     * Get a specific target configuration list
     *
     * @example
     * const project = new Xcode('project.pbxproj');
     *
     * project.getTargets().forEach((tg) => {
     *   const targetConfiguration = project.configuration.getTargetConfigurations(tg.name);
     *
     *   console.log(targetConfiguration);
     * });
     *
     * @param {string} target The name of the target
     * @returns {*} The configuration list (ref) or null if error
     */


    _createClass(Configuration, [{
        key: "getTargetConfigurations",
        value: function getTargetConfigurations(target) {
            var targets = this.xcode.project.objects.PBXNativeTarget;

            for (var i in targets) {
                if (typeof i === "string" && i.indexOf("_comment") === -1) {
                    if (targets[i].name === target) {
                        var configuration = this.xcode.project.objects.XCConfigurationList;

                        return configuration[targets[i].buildConfigurationList];
                    }
                }
            }

            return null;
        }

        /**
         * Get a specific target configuration
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   const targetConfigurationDebug = project.configuration.getTargetConfigurations(tg.name, "Debug");
         *   const targetConfigurationRelease = project.configuration.getTargetConfigurations(tg.name, "Release");
         *
         *   console.log(targetConfigurationDebug);
         *   console.log(targetConfigurationRelease);
         * });
         *
         * @param {string} target The name of the target
         * @param {string} name The name of the configuration
         * @returns {*} The configuration (ref) of null if error
         */

    }, {
        key: "getTargetConfiguration",
        value: function getTargetConfiguration(target, name) {
            var configurations = this.getTargetConfigurations(target);

            if (configurations) {
                var values = configurations.buildConfigurations.map(function (item) {
                    return item.value;
                });

                for (var i in values) {
                    var configuration = this.xcode.project.objects.XCBuildConfiguration[values[i]];
                    if (configuration.name === name) {
                        return configuration;
                    }
                }
            }

            return null;
        }

        /**
         * Get the project configuration list
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         * const projectConfigurations = project.configuration.getProjectConfigurations();
         *
         * console.log(projectConfigurations);
         *
         * @returns {*} The configuration list (ref) or null if error
         */

    }, {
        key: "getProjectConfigurations",
        value: function getProjectConfigurations() {
            var project = Object.keys(this.xcode.project.objects.PBXProject).filter(function (it) {
                return typeof it === "string" && it.indexOf("_comment") === -1;
            });
            if (project && project.length > 0) {
                var projectConfigurationListId = this.xcode.project.objects.PBXProject[project[0]].buildConfigurationList;

                return this.xcode.project.objects.XCConfigurationList[projectConfigurationListId];
            }
            return null;
        }

        /**
         * Get a configuration by name
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         * const projectConfigurationRelease = project.configuration.getProjectConfiguration("Release");
         *
         * console.log(projectConfigurationRelease);
         *
         * @param {string} name The name of the configuration
         * @returns {*} The configuration (ref) or null if error
         */

    }, {
        key: "getProjectConfiguration",
        value: function getProjectConfiguration(name) {
            var configurations = this.getProjectConfigurations();
            var values = configurations.buildConfigurations.map(function (item) {
                return item.value;
            });

            for (var i in values) {
                var configuration = this.xcode.project.objects.XCBuildConfiguration[values[i]];
                if (configuration.name === name) {
                    return configuration;
                }
            }

            return null;
        }

        /**
         * Duplicate a configuration by name (project + all targets)
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.configuration.duplicateConfiguration("Release", "Staging");
         *
         * const projectConfigurationStaging = project.configuration.getProjectConfiguration("Staging");
         *
         * @param {string} oldConfig The name of the configuration to duplicate
         * @param {string} newConfig The new name of duplicated configuraton
         */

    }, {
        key: "duplicateConfiguration",
        value: function duplicateConfiguration(oldConfig, newConfig) {
            var _this = this;

            this.duplicateProjectConfiguration(oldConfig, newConfig);

            var targets = this.xcode.project.objects.PBXNativeTarget;

            var keys = Object.keys(targets).filter(function (it) {
                return typeof it === "string" && it.indexOf("_comment") === -1;
            });

            keys.forEach(function (it) {
                var target = targets[it].name;
                _this.duplicateTargetConfiguration(target, oldConfig, newConfig);
            });
        }

        /**
         * Duplicate a specific target configuration by name
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         * const diplicated = project.configuration.duplicateTargetConfiguration("TestProject", "Release", "Staging");
         *
         * console.log(diplicated);
         *
         * @param {string} target The name of the target
         * @param {string} oldConfig The name of the configuration to duplicate
         * @param {string} newConfig The new name of duplicated configuraton
         * @returns {*} The new duplicated configuration (ref) or null if error
         */

    }, {
        key: "duplicateTargetConfiguration",
        value: function duplicateTargetConfiguration(target, oldConfig, newConfig) {
            var key = _utils2.default.pbx_uid(this.xcode);
            var keyComment = key + "_comment";

            var config = JSON.parse(JSON.stringify(this.getTargetConfiguration(target, oldConfig)));
            config.name = newConfig;

            var configurations = this.getTargetConfigurations(target);
            if (configurations) {
                configurations.buildConfigurations.push({ value: key, comment: newConfig });

                this.xcode.project.objects.XCBuildConfiguration[key] = config;
                this.xcode.project.objects.XCBuildConfiguration[keyComment] = newConfig;

                return this.xcode.project.objects.XCBuildConfiguration[key];
            }

            return null;
        }

        /**
         * Duplicate a project configuration by name (it won't duplicate targets' configurations)
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         * const projectConfigurationStagingResult = project.configuration.duplicateProjectConfiguration("Release", "Staging");
         *
         * console.log(projectConfigurationStagingResult);
         *
         * @param {string} oldConfig The name of the configuration to duplicate
         * @param {string} newConfig The new name of duplicated configuraton
         * @returns {*} The new duplicated configuration (ref) or null if error
         */

    }, {
        key: "duplicateProjectConfiguration",
        value: function duplicateProjectConfiguration(oldConfig, newConfig) {
            var key = _utils2.default.pbx_uid(this.xcode);
            var keyComment = key + "_comment";
            var config = JSON.parse(JSON.stringify(this.getProjectConfiguration(oldConfig)));
            config.name = newConfig;

            var configurations = this.getProjectConfigurations();
            configurations.buildConfigurations.push({ value: key, comment: newConfig });

            this.xcode.project.objects.XCBuildConfiguration[key] = config;
            this.xcode.project.objects.XCBuildConfiguration[keyComment] = newConfig;

            return this.xcode.project.objects.XCBuildConfiguration[key];
        }

        /**
         * Set a user-defined configuration variable for a specific target configuration
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   project.configuration.setUserDefinedTargetConfiguration(tg.name, "Staging" "REACT_HEADERS_PATH", '$(BUILT_PRODUCTS_DIR)/../Release-$(PLATFORM_NAME)/include');
         * });
         *
         * @param {string} target The name of the target
         * @param {string} config The name of the configuration
         * @param {string} key The usef defined variable name
         * @param {string} value The user defined variable value
         */

    }, {
        key: "setUserDefinedTargetConfiguration",
        value: function setUserDefinedTargetConfiguration(target, config, key, value) {
            var conf = this.getTargetConfiguration(target, config);
            var val = "\"" + value + "\"";

            conf.buildSettings[key] = val;
        }

        /**
         * Set a user-defined configuration variable for a specific target (including all configurations)
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   project.configuration.setUserDefinedTarget(tg.name, "REACT_HEADERS_PATH", '$(BUILT_PRODUCTS_DIR)/../Release-$(PLATFORM_NAME)/include');
         * });
         *
         * @param {string} target The name of the target
         * @param {string} config The name of the configuration
         * @param {string} key The usef defined variable name
         * @param {string} value The user defined variable value
         */

    }, {
        key: "setUserDefinedTarget",
        value: function setUserDefinedTarget(target, key, value) {
            var _this2 = this;

            var configs = this.getTargetConfigurations(target);
            var val = "\"" + value + "\"";

            configs && configs.buildConfigurations.forEach(function (it) {
                var conf = _this2.xcode.project.objects.XCBuildConfiguration[it.value];
                conf.buildSettings[key] = val;
            });
        }

        /**
         * Set the headers' search paths for a specific target configuration
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   project.configuration.setHeadersPathTargetConfiguration(tg.name, "Release", ['$(inherited)', '$(REACT_HEADERS_PATH)']);
         * });
         *
         * @param {string} target The name of the target
         * @param {string} config The name of the configuration
         * @param values The headers' search path array
         */

    }, {
        key: "setHeadersPathTargetConfiguration",
        value: function setHeadersPathTargetConfiguration(target, config, values) {
            var conf = this.getTargetConfiguration(target, config);
            var val = values.map(function (it) {
                return "\"" + it + "\"";
            });

            conf.buildSettings['HEADER_SEARCH_PATHS'] = val;
        }

        /**
         * Set the headers' search paths for a specific target (including all configurations)
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   project.configuration.setHeadersPathTarget(tg.name, ['$(inherited)', '$(REACT_HEADERS_PATH)']);
         * });
         *
         * @param {string} target The name of the target
         * @param values The headers' search path array
         */

    }, {
        key: "setHeadersPathTarget",
        value: function setHeadersPathTarget(target, values) {
            var _this3 = this;

            var configs = this.getTargetConfigurations(target);
            var val = values.map(function (it) {
                return "\"" + it + "\"";
            });

            configs && configs.buildConfigurations.forEach(function (it) {
                var conf = _this3.xcode.project.objects.XCBuildConfiguration[it.value];
                conf.buildSettings['HEADER_SEARCH_PATHS'] = val;
            });
        }

        /**
         * Set the preprocessor macros for a specific target configuration
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   project.configuration.setPreprocessorMacrosTargetConfiguration(tg.name, "Release", ['$(inherited)']);
         * });
         *
         * @param {string} target The name of the target
         * @param {string} config The name of the configuration
         * @param values The preprocessor macros array
         */

    }, {
        key: "setPreprocessorMacrosTargetConfiguration",
        value: function setPreprocessorMacrosTargetConfiguration(target, config, values) {
            var conf = this.getTargetConfiguration(target, config);
            var val = values.map(function (it) {
                return "\"" + it + "\"";
            });

            conf.buildSettings['GCC_PREPROCESSOR_DEFINITIONS'] = val;
        }

        /**
         * Set the preprocessor macros for a specific target (including all configurations)
         *
         * @example
         * const project = new Xcode('project.pbxproj');
         *
         * project.getTargets().forEach((tg) => {
         *   project.configuration.setPreprocessorMacrosTarget(tg.name, ['$(inherited)']);
         * });
         *
         * @param {string} target The name of the target
         * @param values The preprocessor macros array
         */

    }, {
        key: "setPreprocessorMacrosTarget",
        value: function setPreprocessorMacrosTarget(target, values) {
            var _this4 = this;

            var configs = this.getTargetConfigurations(target);
            var val = values.map(function (it) {
                return "\"" + it + "\"";
            });

            configs && configs.buildConfigurations.forEach(function (it) {
                var conf = _this4.xcode.project.objects.XCBuildConfiguration[it.value];
                conf.buildSettings['GCC_PREPROCESSOR_DEFINITIONS'] = val;
            });
        }
    }]);

    return Configuration;
}();

exports.default = Configuration;